#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Dec 18 11:59:46 2024

Erstellung eines Test-Datensatzes für Klassifizierungsaufgaben
==============================================================

Der Testdatensatz wird unter Vorgabe der Dimensionen, der Klassen und der
Punkte pro Klasse erzeugt. 

Die Punkte jeder Klasse können mit einem ZUfallswert variiert werden. 
Datensätze der Dimensionen 1 - 3 können visualisiert werden. Die notwendigen,
nicht im Datensatz enthaltenen Daten werden im Modul zwischengespeichert.

Besondere Prüfungen nicht statt. Der Anwender sollte wissen, wie er die
Methoden benutzt

@author: user
"""
import random as rd
import matplotlib.pyplot as plt
import math
import pickle

#
# Satz von globalen Variablen mit den Feldkenngrößen
# 
ddim = 0
dkls = 0
dpts = 0


#
# Initiiert ein Feld mit dim Dimensionen, kls Klassen und jeweils pts geleichen Werten
# mit Hilfe von Zufallzahlen im Bereich 0 - 50 
#
def InitSet(dim,kls,pts):
    
    pc = []
    global ddim
    ddim = dim
    global dkls
    dkls = kls
    global dpts
    dpts = pts   
    for j in range(kls):
        pd = []
        for i in range(dim):
            pd.append(rd.randrange(50))
        for i in range(pts): 
            pc.append(pd)

    return pc



#
# Plottet Felder in den Dimensionen 1 - 3
#
def PlotSet(a):    
    colors = ['bo', 'go', 'ro', 'co', 'mo', 'yo', 'ko','bo', 'go', 'ro', 'co', 'mo', 'yo', 'ko']
    
    al=len(a)       
    d=len(a[0])
    plt.axis([-2,52,-2,52])
    plt.grid(True)
    if(d == 1):
        for i in range(al):
            plt.plot(a[i],colors[i//dpts])
#        plt.show()    
    elif(d == 2):        
        for i in range(al):
            plt.plot(a[i][0],a[i][1],colors[i//dpts])
#        plt.show()    
    elif(d == 3):        
        pl = plt.figure().add_subplot(projection='3d')
        for i in range(al):
            pl.plot(a[i][0],a[i][1],a[i][2],colors[i//dpts])
    else:
        print("ZU VIELE DIMENSIONEN, PLOT NICHT MÖGLICH")
            

#
# Plotset mit Gruppen
# Die zu einer Gruppe gehörenden Punkte werden durch einen Kreis gekennzeichnet,
# der um den Schwerpunkt der Punktgruppe gezogen wird.
#
def GroupPlotSet(a,mid,r):
    fig = plt.figure()
    ax = fig.add_subplot()
    for i in range(len(mid)):
        circle = plt.Circle((mid[i][0],mid[i][1]), r, clip_on=False, fill=False , color='r')
        ax.add_patch(circle)
    PlotSet(a)




        
#
# Modifiziert die Werte der Klassen kls mit normalverteilten Zufallsdaten         
# der Streuung r
#
def ModifyClass(a,kls,r):
    for i in range(dpts):
        b=[]
        for j in range(ddim):
            b.append(a[i+dpts*kls][j] + rd.normalvariate(0.0, r))
        a[i+dpts*kls]=b
    return a;            



#
# Sichern und Rücklesen einer Liste in eine Datei
#
def WriteList(a,name):
    global ddim
    global dkls
    global dpts
    with open(name, "wb") as fp:
        pickle.dump(a, fp)
        pickle.dump(ddim,fp)
        pickle.dump(dkls,fp)
        pickle.dump(dpts,fp)
        
        
def ReadList(name):
    global ddim
    global dkls
    global dpts
    with open(name, "rb") as fp:   # Unpickling
        a = pickle.load(fp)        
        ddim = pickle.load(fp)
        dkls = pickle.load(fp)
        dpts = pickle.load(fp)
    return a